@@ -51,6 +51,7 @@ extern void initregex();
 extern void initrgbimg();
 extern void initrotor();
 extern void initsignal();
+extern void initsha();
 extern void initselect();
 extern void init_socket();
 extern void initsoundex();
@@ -63,6 +64,8 @@ extern void initcPickle();
 extern void initpcre();
 #ifdef WIN32
 extern void initmsvcrt();
+extern void initwinsound();
+extern void init_locale();
 #endif
 
 /* -- ADDMODULE MARKER 1 -- */
@@ -73,7 +76,7 @@ extern void initimp();
 struct _inittab _PyImport_Inittab[] = {
 
         {"array", initarray},
-#ifdef M_I386
+#ifdef MS_WINDOWS
         {"audioop", initaudioop},
 #endif
         {"binascii", initbinascii},
@@ -89,6 +92,7 @@ struct _inittab _PyImport_Inittab[] = {
         {"rgbimg", initrgbimg},
         {"rotor", initrotor},
         {"signal", initsignal},
+        {"sha", initsha},
 #ifdef USE_SOCKET
         {"_socket", init_socket},
         {"select", initselect},
@@ -105,6 +109,8 @@ struct _inittab _PyImport_Inittab[] = {
         {"pcre", initpcre},
 #ifdef WIN32
 	{"msvcrt", initmsvcrt},
+	{"winsound", initwinsound},
+	{"_locale", init_locale},
 #endif
 
 /* -- ADDMODULE MARKER 2 -- */
