@@ -46,7 +46,8 @@ PERFORMANCE OF THIS SOFTWARE.
    symbol	-- defined for:
 
    DYNAMIC_LINK -- any kind of dynamic linking
-   USE_RLD	-- NeXT dynamic linking
+   USE_RLD	-- NeXT dynamic linking with FVM shlibs
+   USE_DYLD     -- NeXT dynamic linking with frameworks
    USE_DL	-- Jack's dl for IRIX 4 or GNU dld with emulation for Jack's dl
    USE_SHLIB	-- SunOS or IRIX 5 (SVR4?) shared libraries
    _AIX		-- AIX style dynamic linking
@@ -61,6 +62,7 @@ PERFORMANCE OF THIS SOFTWARE.
    __NetBSD__	-- NetBSD shared libraries
 		   (assuming dlerror() was introduced between 1.2 and 1.3)
    __FreeBSD__	-- FreeBSD shared libraries
+   __BEOS__ -- BeOS shared libraries - defined by the compiler
 
    (The other WITH_* symbols are used only once, to set the
    appropriate symbols.)
@@ -104,15 +106,41 @@ typedef int (* APIENTRY dl_funcptr)();
 #ifdef MS_WINDOWS /* i.e. MS_WIN32 or MS_WIN16 */
 #define DYNAMIC_LINK
 #include <windows.h>
+#include <direct.h>
 typedef FARPROC dl_funcptr;
 #define _DL_FUNCPTR_DEFINED
+#ifdef _DEBUG
+#define SHORT_EXT "_d.pyd"
+#define LONG_EXT "_d.dll"
+#else
 #define SHORT_EXT ".pyd"
 #define LONG_EXT ".dll"
 #endif
+#endif
 
-#ifdef NeXT
+#ifdef WITH_DYLD
+#define DYNAMIC_LINK
+#define USE_DYLD
+#define SHORT_EXT ".so"
+#define LONG_EXT "module.so"
+#define FUNCNAME_PATTERN "_init%.200s"
+#endif
+
+#if defined(NeXT) && !defined(DYNAMIC_LINK)
 #define DYNAMIC_LINK
 #define USE_RLD
+/* Define this to 1 if you want be able to load ObjC modules as well:
+   it switches between two different way of loading modules on the NeXT,
+   one that directly interfaces with the dynamic loader (rld_load(), which
+   does not correctly load ObjC object files), and another that uses the
+   ObjC runtime (objc_loadModules()) to do the job.
+   You'll have to add ``-ObjC'' to the compiler flags if you set this to 1.
+*/
+#define HANDLE_OBJC_MODULES 1
+#if HANDLE_OBJC_MODULES
+#include <objc/Object.h>
+#include <objc/objc-load.h>
+#endif
 #define SHORT_EXT ".so"
 #define LONG_EXT "module.so"
 #endif
@@ -158,6 +186,36 @@ static int  aix_bindnewmodule(void *, void *);
 static void aix_loaderror(char *);
 #endif
 
+#ifdef __BEOS__
+#undef USE_SHLIB /* probably not defined anyway */
+#define DYNAMIC_LINK
+#define SHORT_EXT ".so"
+#define LONG_EXT "module.so"
+typedef void (*dl_funcptr)(void);
+#define _DL_FUNCPTR_DEFINED
+
+#if defined(MAXPATHLEN) && !defined(_SYS_PARAM_H)
+#undef MAXPATHLEN
+#endif
+
+#include <kernel/image.h>
+#include <kernel/OS.h>
+#include <stdlib.h>
+#include <unistd.h>
+
+#ifdef WITH_THREAD
+#include "pythread.h"
+static PyThread_type_lock beos_dyn_lock;
+#endif
+
+static PyObject *beos_dyn_images = NULL;
+
+static void beos_init_dyn( void );
+static void beos_cleanup_dyn( void );
+static void beos_nuke_dyn( PyObject *item );
+static void beos_add_dyn( char *pathname, image_id id );
+#endif
+
 #ifdef DYNAMIC_LINK
 
 #ifdef USE_SHLIB
@@ -177,9 +235,6 @@ typedef void (*dl_funcptr)();
 #ifndef RTLD_LAZY
 #define RTLD_LAZY 1
 #endif
-#ifndef RTLD_GLOBAL
-#define RTLD_GLOBAL 0
-#endif
 #define SHORT_EXT ".so"
 #define LONG_EXT "module.so"
 #endif /* USE_SHLIB */
@@ -195,6 +250,9 @@ typedef void (*dl_funcptr)();
 #define CFragConnectionID ConnectionID
 #define kLoadCFrag 0x01
 #endif
+#ifdef USE_GUSI
+#include "TFileSpec.h"		/* for Path2FSSpec() */
+#endif
 #include <Files.h>
 #include "macdefs.h"
 #include "macglue.h"
@@ -202,16 +260,22 @@ typedef void (*dl_funcptr)();
 
 #ifdef USE_RLD
 #include <mach-o/rld.h>
-#define FUNCNAME_PATTERN "_init%.200s"
 #ifndef _DL_FUNCPTR_DEFINED
 typedef void (*dl_funcptr)();
 #endif
 #endif /* USE_RLD */
 
+#ifdef USE_DYLD
+#include <mach-o/dyld.h>
+#ifndef _DL_FUNCPTR_DEFINED
+typedef void (*dl_funcptr)();
+#endif
+#endif /* USE_DYLD */
+
 extern char *Py_GetProgramName();
 
 #ifndef FUNCNAME_PATTERN
-#if defined(__hp9000s300) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__BORLANDC__)
+#if defined(__hp9000s300) || (defined(__NetBSD__) || defined(__FreeBSD__)) && !defined(__ELF__) || defined(__OpenBSD__) || defined(__BORLANDC__) || defined(NeXT)
 #define FUNCNAME_PATTERN "_init%.200s"
 #else
 #define FUNCNAME_PATTERN "init%.200s"
@@ -318,15 +382,21 @@ _PyImport_LoadDynamicModule(name, pathname, fp)
 		Ptr mainAddr;
 		Str255 errMessage;
 		OSErr err;
+#ifndef USE_GUSI
 		Boolean isfolder, didsomething;
+#endif
 		char buf[512];
 		Str63 fragname;
 		Ptr symAddr;
 		CFragSymbolClass class;
 		
 		/* First resolve any aliases to find the real file */
+#ifdef USE_GUSI
+		err = Path2FSSpec(pathname, &libspec);
+#else
 		(void)FSMakeFSSpec(0, 0, Pstring(pathname), &libspec);
 		err = ResolveAliasFile(&libspec, 1, &isfolder, &didsomething);
+#endif
 		if ( err ) {
 			sprintf(buf, "%.255s: %.200s",
 				pathname, PyMac_StrError(err));
@@ -371,13 +441,13 @@ _PyImport_LoadDynamicModule(name, pathname, fp)
 #ifdef RTLD_NOW
 		/* RTLD_NOW: resolve externals now
 		   (i.e. core dump now if some are missing) */
-		void *handle = dlopen(pathname, RTLD_NOW | RTLD_GLOBAL);
+		void *handle = dlopen(pathname, RTLD_NOW);
 #else
 		void *handle;
 		if (Py_VerboseFlag)
 			printf("dlopen(\"%s\", %d);\n", pathname,
-			       RTLD_LAZY | RTLD_GLOBAL);
-		handle = dlopen(pathname, RTLD_LAZY | RTLD_GLOBAL);
+			       RTLD_LAZY);
+		handle = dlopen(pathname, RTLD_LAZY);
 #endif /* RTLD_NOW */
 		if (handle == NULL) {
 			PyErr_SetString(PyExc_ImportError, dlerror());
@@ -417,7 +487,23 @@ _PyImport_LoadDynamicModule(name, pathname, fp)
 #ifdef MS_WIN32
 	{
 		HINSTANCE hDLL;
-		hDLL = LoadLibrary(pathname);
+		char pathbuf[260];
+		if (strchr(pathname, SEP) == NULL &&
+		    strchr(pathname, ALTSEP) == NULL)
+		{
+			/* Prefix bare filename with ".\" */
+			char *p = pathbuf;
+			*p = '\0';
+			_getcwd(pathbuf, sizeof pathbuf);
+			if (*p != '\0' && p[1] == ':')
+				p += 2;
+			sprintf(p, ".\\%-.255s", pathname);
+			pathname = pathbuf;
+		}
+		/* Look for dependent DLLs in directory of pathname first */
+		/* XXX This call doesn't exist in Windows CE */
+		hDLL = LoadLibraryEx(pathname, NULL,
+				     LOAD_WITH_ALTERED_SEARCH_PATH);
 		if (hDLL==NULL){
 			char errBuf[256];
 			unsigned int errorCode;
@@ -469,6 +555,14 @@ _PyImport_LoadDynamicModule(name, pathname, fp)
 #ifdef MS_WIN16
 	{
 		HINSTANCE hDLL;
+		char pathbuf[16];
+		if (strchr(pathname, SEP) == NULL &&
+		    strchr(pathname, ALTSEP) == NULL)
+		{
+			/* Prefix bare filename with ".\" */
+			sprintf(pathbuf, ".\\%-.13s", pathname);
+			pathname = pathbuf;
+		}
 		hDLL = LoadLibrary(pathname);
 		if (hDLL < HINSTANCE_ERROR){
 			char errBuf[256];
@@ -485,24 +579,25 @@ _PyImport_LoadDynamicModule(name, pathname, fp)
 	{
 		APIRET  rc;
 		HMODULE hDLL;
-        char failreason[256];
+		char failreason[256];
 
 		rc = DosLoadModule(failreason,
-                           sizeof(failreason),
-                           pathname,
-                           &hDLL);
+				   sizeof(failreason),
+				   pathname,
+				   &hDLL);
 
 		if (rc != NO_ERROR) {
 			char errBuf[256];
 			sprintf(errBuf,
-				"DLL load failed, rc = %d, problem '%s': %s", rc, failreason);
+				"DLL load failed, rc = %d, problem '%s': %s",
+				rc, failreason);
 			PyErr_SetString(PyExc_ImportError, errBuf);
 			return NULL;
 		}
 
-        rc = DosQueryProcAddr(hDLL, 0L, funcname, &p);
-        if (rc != NO_ERROR)
-            p = NULL; /* Signify Failure to Acquire Entrypoint */
+		rc = DosQueryProcAddr(hDLL, 0L, funcname, &p);
+		if (rc != NO_ERROR)
+			p = NULL; /* Signify Failure to Acquire Entrypoint */
 	}
 #endif /* PYOS_OS2 */
 
@@ -520,9 +615,38 @@ _PyImport_LoadDynamicModule(name, pathname, fp)
 		errorStream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
 		filenames[0] = pathname;
 		filenames[1] = NULL;
+
+#if HANDLE_OBJC_MODULES
+
+/* The following very bogus line of code ensures that
+   objc_msgSend, etc are linked into the binary.  Without
+   it, dynamic loading of a module that includes objective-c
+   method calls will fail with "undefined symbol _objc_msgSend()".
+   This remains true even in the presence of the -ObjC flag
+   to the compiler
+*/
+
+		[Object name];
+
+/* objc_loadModules() dynamically loads the object files
+   indicated by the paths in filenames.  If there are any
+   errors generated during loading -- typically due to the
+   inability to find particular symbols -- an error message
+   will be written to errorStream.
+   It returns 0 if the module is successfully loaded, 1
+   otherwise.
+*/
+
+		ret = !objc_loadModules(filenames, errorStream,
+					NULL, &new_header, NULL);
+
+#else /* !HANDLE_OBJC_MODULES */
+
 		ret = rld_load(errorStream, &new_header, 
 				filenames, NULL);
 
+#endif /* HANDLE_OBJC_MODULES */
+
 		/* extract the error messages for the exception */
 		if(!ret) {
 			char *streamBuf;
@@ -544,35 +668,176 @@ _PyImport_LoadDynamicModule(name, pathname, fp)
 			return NULL;
 	}
 #endif /* USE_RLD */
+#ifdef USE_DYLD
+	/* This is also NeXT-specific. However, frameworks (the new style
+	of shared library) and rld() can't be used in the same program;
+	instead, you have to use dyld, which is mostly unimplemented. */
+	{
+		NSObjectFileImageReturnCode rc;
+		NSObjectFileImage image;
+		NSModule newModule;
+		NSSymbol theSym;
+		void *symaddr;
+		const char *errString;
+	
+		rc = NSCreateObjectFileImageFromFile(pathname, &image);
+		switch(rc) {
+		    default:
+		    case NSObjectFileImageFailure:
+		    NSObjectFileImageFormat:
+		    /* for these a message is printed on stderr by dyld */
+			errString = "Can't create object file image";
+			break;
+		    case NSObjectFileImageSuccess:
+			errString = NULL;
+			break;
+		    case NSObjectFileImageInappropriateFile:
+			errString = "Inappropriate file type for dynamic loading";
+			break;
+		    case NSObjectFileImageArch:
+			errString = "Wrong CPU type in object file";
+			break;
+		    NSObjectFileImageAccess:
+			errString = "Can't read object file (no access)";
+			break;
+		}
+		if (errString == NULL) {
+			newModule = NSLinkModule(image, pathname, TRUE);
+			if (!newModule)
+				errString = "Failure linking new module";
+		}
+		if (errString != NULL) {
+			PyErr_SetString(PyExc_ImportError, errString);
+			return NULL;
+		}
+		if (!NSIsSymbolNameDefined(funcname)) {
+			/* UnlinkModule() isn't implimented in current versions, but calling it does no harm */
+			NSUnLinkModule(newModule, FALSE);
+			PyErr_Format(PyExc_ImportError, "Loaded module does not contain symbol %s", funcname);
+			return NULL;
+		}
+		theSym = NSLookupAndBindSymbol(funcname);
+		p = (dl_funcptr)NSAddressOfSymbol(theSym);
+ 	}
+#endif /* USE_DYLD */
 #ifdef hpux
 	{
 		shl_t lib;
 		int flags;
 
 		flags = BIND_FIRST | BIND_DEFERRED;
-		if (Py_VerboseFlag)
-                {
-                        flags = DYNAMIC_PATH | BIND_FIRST | BIND_IMMEDIATE |
+		if (Py_VerboseFlag) {
+			flags = DYNAMIC_PATH | BIND_FIRST | BIND_IMMEDIATE |
 				BIND_NONFATAL | BIND_VERBOSE;
-                        printf("shl_load %s\n",pathname);
-                }
-                lib = shl_load(pathname, flags, 0);
-                if (lib == NULL)
-                {
-                        char buf[256];
-                        if (Py_VerboseFlag)
-                                perror(pathname);
-                        sprintf(buf, "Failed to load %.200s", pathname);
-                        PyErr_SetString(PyExc_ImportError, buf);
-                        return NULL;
-                }
-                if (Py_VerboseFlag)
-                        printf("shl_findsym %s\n", funcname);
-                shl_findsym(&lib, funcname, TYPE_UNDEFINED, (void *) &p);
-                if (p == NULL && Py_VerboseFlag)
-                        perror(funcname);
+			printf("shl_load %s\n",pathname);
+		}
+		lib = shl_load(pathname, flags, 0);
+		/* XXX Chuck Blake once wrote that 0 should be BIND_NOSTART? */
+		if (lib == NULL) {
+			char buf[256];
+			if (Py_VerboseFlag)
+				perror(pathname);
+			sprintf(buf, "Failed to load %.200s", pathname);
+			PyErr_SetString(PyExc_ImportError, buf);
+			return NULL;
+		}
+		if (Py_VerboseFlag)
+			printf("shl_findsym %s\n", funcname);
+		shl_findsym(&lib, funcname, TYPE_UNDEFINED, (void *) &p);
+		if (p == NULL && Py_VerboseFlag)
+			perror(funcname);
 	}
 #endif /* hpux */
+#ifdef __BEOS__
+	{
+		image_id the_id;
+		status_t retval;
+		char fullpath[PATH_MAX];
+
+		if( Py_VerboseFlag ) {
+			printf( "load_add_on( %s )\n", pathname );
+		}
+
+		/* Hmm, this old bug appears to have regenerated itself; if the
+		 * path isn't absolute, load_add_on() will fail.  Reported to Be
+		 * April 21, 1998.
+		 */
+		if( pathname[0] != '/' ) {
+			(void)getcwd( fullpath, PATH_MAX );
+			(void)strncat( fullpath, "/", PATH_MAX );
+			(void)strncat( fullpath, pathname, PATH_MAX );
+			
+			if( Py_VerboseFlag ) {
+				printf( "load_add_on( %s )\n", fullpath );
+			}
+		} else {
+			(void)strcpy( fullpath, pathname );
+		}
+
+		the_id = load_add_on( fullpath );
+		if( the_id < B_NO_ERROR ) {
+			/* It's too bad load_add_on() doesn't set errno or something...
+			 */
+			char buff[256];  /* hate hard-coded string sizes... */
+
+			if( Py_VerboseFlag ) {
+				printf( "load_add_on( %s ) failed", fullpath );
+			}
+
+			switch( the_id ) {
+			case B_ERROR:
+				sprintf( buff, "BeOS: Failed to load %.200s", fullpath );
+				break;
+			default:
+				sprintf( buff, "Unknown error loading %.200s", fullpath );
+				break;
+			}
+
+			PyErr_SetString( PyExc_ImportError, buff );
+			return NULL;
+		}
+
+		if( Py_VerboseFlag ) {
+			printf( "get_image_symbol( %s )\n", funcname );
+		}
+
+		retval = get_image_symbol( the_id, funcname, B_SYMBOL_TYPE_TEXT, &p );
+		if( retval != B_NO_ERROR || p == NULL ) {
+			/* That's bad, we can't find that symbol in the module...
+			 */
+			char buff[256];  /* hate hard-coded string sizes... */
+
+			if( Py_VerboseFlag ) {
+				printf( "get_image_symbol( %s ) failed", funcname );
+			}
+
+			switch( retval ) {
+			case B_BAD_IMAGE_ID:
+				sprintf( buff, "can't load init function for dynamic module: "
+				               "Invalid image ID for %.180s", fullpath );
+				break;
+			case B_BAD_INDEX:
+				sprintf( buff, "can't load init function for dynamic module: "
+				               "Bad index for %.180s", funcname );
+				break;
+			default:
+				sprintf( buff, "can't load init function for dynamic module: "
+				               "Unknown error looking up %.180s", funcname );
+				break;
+			}
+
+			retval = unload_add_on( the_id );
+
+			PyErr_SetString( PyExc_ImportError, buff );
+			return NULL;
+		}
+
+		/* Save the module name and image ID for later so we can clean up
+		 * gracefully.
+		 */
+		beos_add_dyn( name, the_id );
+	}
+#endif /* __BEOS__ */
 #ifdef USE_SHLIB
   got_it:
 #endif
@@ -603,7 +868,7 @@ _PyImport_LoadDynamicModule(name, pathname, fp)
 		PyErr_Clear(); /* Not important enough to report */
 	Py_XDECREF(s);
 	if (Py_VerboseFlag)
-		fprintf(stderr,
+		PySys_WriteStderr(
 			"import %s # dynamically loaded from %s\n",
 			name, pathname);
 	Py_INCREF(m);
@@ -761,3 +1026,133 @@ aix_loaderror(pathname)
 }
 
 #endif /* _AIX */
+
+#ifdef __BEOS__
+/* ----------------------------------------------------------------------
+ * BeOS dynamic loading support
+ *
+ * This uses shared libraries, but BeOS has its own way of doing things
+ * (much easier than dlfnc.h, from the look of things).  We'll use a
+ * Python Dictionary object to store the images_ids so we can be very
+ * nice and unload them when we exit.
+ *
+ * Note that this is thread-safe.  Probably irrelevent, because of losing
+ * systems... Python probably disables threads while loading modules.
+ * Note the use of "probably"!  Better to be safe than sorry. [chrish]
+ *
+ * As of 1.5.1 this should also work properly when you've configured
+ * Python without thread support; the 1.5 version required it, which wasn't
+ * very friendly.  Note that I haven't tested it without threading... why
+ * would you want to avoid threads on BeOS? [chrish]
+ *
+ * As of 1.5.2, the PyImport_BeImageID() function has been removed; Donn
+ * tells me it's not necessary anymore because of PyCObject_Import().
+ * [chrish]
+ */
+
+/*
+ * Initialize our dictionary, and the dictionary mutex.
+ */
+static void beos_init_dyn( void )
+{
+	/* We're protected from a race condition here by the atomic init_count
+	 * variable.
+	 */
+	static int32 init_count = 0;
+	int32 val;
+
+	val = atomic_add( &init_count, 1 );
+	if( beos_dyn_images == NULL && val == 0 ) {
+		beos_dyn_images = PyDict_New();
+#ifdef WITH_THREAD
+		beos_dyn_lock = PyThread_allocate_lock();
+#endif
+		atexit( beos_cleanup_dyn );
+	}
+}
+
+/* atexit() handler that'll call unload_add_on() for every item in the
+ * dictionary.
+ */
+static void beos_cleanup_dyn( void )
+{
+	if( beos_dyn_images ) {
+		int idx;
+		int list_size;
+		PyObject *id_list;
+
+#ifdef WITH_THREAD
+		PyThread_acquire_lock( beos_dyn_lock, 1 );
+#endif
+
+		id_list = PyDict_Values( beos_dyn_images );
+
+		list_size = PyList_Size( id_list );
+		for( idx = 0; idx < list_size; idx++ ) {
+			PyObject *the_item;
+			
+			the_item = PyList_GetItem( id_list, idx );
+			beos_nuke_dyn( the_item );
+		}
+
+		PyDict_Clear( beos_dyn_images );
+
+#ifdef WITH_THREAD
+		PyThread_free_lock( beos_dyn_lock );
+#endif
+	}
+}
+
+/* Whack an item; the item is an image_id in disguise, so we'll call
+ * unload_add_on() for it.
+ */
+static void beos_nuke_dyn( PyObject *item )
+{
+	status_t retval;
+
+	if( item ) {
+		image_id id = (image_id)PyInt_AsLong( item );
+		
+		retval = unload_add_on( id );
+	}
+}
+
+/*
+ * Add an image_id to the dictionary; the module name of the loaded image
+ * is the key.  Note that if the key is already in the dict, we unload
+ * that image; this should allow reload() to work on dynamically loaded
+ * modules (super-keen!).
+ */
+static void beos_add_dyn( char *name, image_id id )
+{
+	int retval;
+	PyObject *py_id;
+
+	if( beos_dyn_images == NULL ) {
+		beos_init_dyn();
+	}
+
+#ifdef WITH_THREAD
+	retval = PyThread_acquire_lock( beos_dyn_lock, 1 );
+#endif
+
+	/* If there's already an object with this key in the dictionary,
+	 * we're doing a reload(), so let's nuke it.
+	 */
+	py_id = PyDict_GetItemString( beos_dyn_images, name );
+	if( py_id ) {
+		beos_nuke_dyn( py_id );
+		retval = PyDict_DelItemString( beos_dyn_images, name );
+	}
+
+	py_id = PyInt_FromLong( (long)id );
+	if( py_id ) {
+		retval = PyDict_SetItemString( beos_dyn_images, name, py_id );
+	}
+
+#ifdef WITH_THREAD
+	PyThread_release_lock( beos_dyn_lock );
+#endif
+}
+
+#endif /* __BEOS__ */
