@@ -291,6 +291,11 @@ do_mkvalue(p_format, p_va)
 		case 'l':
 			return PyInt_FromLong((long)va_arg(*p_va, long));
 
+#ifdef HAVE_LONG_LONG
+		case 'L':
+			return PyLong_FromLongLong((LONG_LONG)va_arg(*p_va, LONG_LONG));
+#endif
+
 		case 'f':
 		case 'd':
 			return PyFloat_FromDouble(
@@ -327,6 +332,7 @@ do_mkvalue(p_format, p_va)
 			return v;
 		}
 
+		case 'N':
 		case 'S':
 		case 'O':
 		if (**p_format == '&') {
@@ -339,8 +345,10 @@ do_mkvalue(p_format, p_va)
 		else {
 			PyObject *v;
 			v = va_arg(*p_va, PyObject *);
-			if (v != NULL)
-				Py_INCREF(v);
+			if (v != NULL) {
+				if (*(*p_format - 1) != 'N')
+					Py_INCREF(v);
+			}
 			else if (!PyErr_Occurred())
 				/* If a NULL was passed
 				 * because a call that should
@@ -351,7 +359,7 @@ do_mkvalue(p_format, p_va)
 				 * clear that the caller knew
 				 * what she was doing. */
 				PyErr_SetString(PyExc_SystemError,
-					   "NULL object passed to mkvalue");
+					"NULL object passed to Py_BuildValue");
 			return v;
 		}
 
@@ -363,7 +371,7 @@ do_mkvalue(p_format, p_va)
 
 		default:
 			PyErr_SetString(PyExc_SystemError,
-				   "bad format char passed to mkvalue");
+				"bad format char passed to Py_BuildValue");
 			return NULL;
 
 		}
@@ -456,12 +464,12 @@ PyEval_CallFunction(obj, format, va_alist)
 
 #ifdef HAVE_STDARG_PROTOTYPES
 PyObject *
-PyEval_CallMethod(PyObject *obj, char *methonname, char *format, ...)
+PyEval_CallMethod(PyObject *obj, char *methodname, char *format, ...)
 #else
 PyObject *
-PyEval_CallMethod(obj, methonname, format, va_alist)
+PyEval_CallMethod(obj, methodname, format, va_alist)
 	PyObject *obj;
-	char *methonname;
+	char *methodname;
 	char *format;
 	va_dcl
 #endif
@@ -471,7 +479,7 @@ PyEval_CallMethod(obj, methonname, format, va_alist)
 	PyObject *args;
 	PyObject *res;
 
-	meth = PyObject_GetAttrString(obj, methonname);
+	meth = PyObject_GetAttrString(obj, methodname);
 	if (meth == NULL)
 		return NULL;
 
